require 'od/payments/version'
require 'od/payments/errors'
require 'od/payments/resources'
require 'stripe'
require 'square'
module Od
  module Payments
    class << self
      ADAPTERS = %i[stripe square].freeze
      attr_accessor :api_key_stripe
      attr_accessor :access_token_square
      attr_accessor :environment_square
      attr_accessor :client_square
      attr_accessor :production

      def configure(adapter, options = {})
        self.adapter = adapter
        @api_key_stripe = options[:api_key_stripe]
        @access_token_square = options[:access_token_square]
        @environment_square = options[:environment_square]
        @production = options[:production]
        initializate_stripe if validate_stripe
        initializate_square if validate_square
      end

      def validate_stripe
        !api_key_stripe.nil? && !api_key_stripe&.strip&.empty?
      end

      def validate_square
        !access_token_square.nil? || !access_token_square&.strip&.empty?
      end

      def initializate_stripe
        Stripe.api_key = api_key_stripe
      end

      def initializate_square
        @client_square = Square::Client.new(
          access_token: access_token_square,
          environment: production ? 'production' : 'sandbox'
      )
      end

      def adapter
        return @adapter if @adapter

        self.adapter = :stripe
        @adapter
      end

      def adapter=(adapter)
        raise Od::Payments::APIError, 'Invalid Adapter' if adapter.class != Symbol || !ADAPTERS.include?(adapter)

        @adapter = adapter
      end
    end
  end
end
