module Od
  module Payments
    module PaymentMethod

      def self.create(params)
        case Od::Payments.adapter
        when :stripe
          Stripe::PaymentMethod.create params
        end
      rescue Stripe::CardError => e
        raise Od::Payments::CardError.new(e.error.message, {})
      rescue Stripe::StripeError => e
        raise Od::Payments::OdPaymentsError, e.error.message
      end

      def self.list(customer_id)
        case Od::Payments.adapter
        when :stripe
          Stripe::PaymentMethod.list({
                                       customer: customer_id,
                                       type: 'card'
                                     })
        end
      rescue Stripe::StripeError => e
        raise Od::Payments::OdPaymentsError, e.error.message
      end

      def self.delete(payment_method_id)
        case Od::Payments.adapter
        when :stripe
          Stripe::PaymentMethod.detach(
            payment_method_id
          )
        end
      rescue Stripe::StripeError => e
        raise Od::Payments::OdPaymentsError, e.error.message
      end

      def self.show(payment_method_id)
        case Od::Payments.adapter
        when :stripe
          Stripe::PaymentIntent.retrieve payment_method_id
        end
      rescue Stripe::StripeError => e
        raise Od::Payments::OdPaymentsError, e.error.message
      end

      def self.attach(payment_method_id, customer_id)
        Stripe::PaymentMethod.attach(
          payment_method_id,
          { customer: customer_id }
        )
      rescue Stripe::CardError => e
        raise Od::Payments::CardError.new(e.error.message, {})
      rescue Stripe::StripeError => e
        raise Od::Payments::OdPaymentsError, e.error.message
      end
    end
  end
end
